/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.CodeActionContext;
import io.typefox.lsapi.CodeActionParams;
import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.TextDocumentIdentifier;
import io.typefox.lsapi.builders.CodeActionContextBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.RangeBuilder;
import io.typefox.lsapi.builders.TextDocumentIdentifierBuilder;
import io.typefox.lsapi.impl.CodeActionContextImpl;
import io.typefox.lsapi.impl.CodeActionParamsImpl;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link CodeActionParams}.
 */
@SuppressWarnings("all")
public class CodeActionParamsBuilder implements IBuilder<CodeActionParams> {
  public CodeActionParamsBuilder() {
    
  }
  
  public CodeActionParamsBuilder(final Procedure1<CodeActionParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The document in which the command was invoked.
   */
  private TextDocumentIdentifierImpl textDocument;
  
  public CodeActionParamsBuilder textDocument(final TextDocumentIdentifier textDocument) {
    if (textDocument != null && !(textDocument instanceof TextDocumentIdentifierImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (TextDocumentIdentifierImpl) textDocument;
    return this;
  }
  
  public CodeActionParamsBuilder textDocument(final Procedure1<TextDocumentIdentifierBuilder> initializer) {
    TextDocumentIdentifierBuilder builder = new TextDocumentIdentifierBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  public CodeActionParamsBuilder textDocument(final String uri) {
    TextDocumentIdentifier instance = new TextDocumentIdentifierImpl(uri);
    return textDocument(instance);
  }
  
  /**
   * The range for which the command was invoked.
   */
  private RangeImpl range;
  
  public CodeActionParamsBuilder range(final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    this.range = (RangeImpl) range;
    return this;
  }
  
  public CodeActionParamsBuilder range(final Procedure1<RangeBuilder> initializer) {
    RangeBuilder builder = new RangeBuilder();
    initializer.apply(builder);
    return range(builder.build());
  }
  
  public CodeActionParamsBuilder range(final Position start, final Position end) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    Range instance = new RangeImpl((PositionImpl) start, (PositionImpl) end);
    return range(instance);
  }
  
  /**
   * Context carrying additional information.
   */
  private CodeActionContextImpl context;
  
  public CodeActionParamsBuilder context(final CodeActionContext context) {
    if (context != null && !(context instanceof CodeActionContextImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + context.getClass().getSimpleName());
    this.context = (CodeActionContextImpl) context;
    return this;
  }
  
  public CodeActionParamsBuilder context(final Procedure1<CodeActionContextBuilder> initializer) {
    CodeActionContextBuilder builder = new CodeActionContextBuilder();
    initializer.apply(builder);
    return context(builder.build());
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public CodeActionParams build() {
    CodeActionParamsImpl result = new CodeActionParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final CodeActionParamsImpl result) {
    result.setTextDocument(this.textDocument);
    result.setRange(this.range);
    result.setContext(this.context);
  }
}
