/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.CodeLens;
import io.typefox.lsapi.Command;
import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.builders.CommandBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.RangeBuilder;
import io.typefox.lsapi.impl.CodeLensImpl;
import io.typefox.lsapi.impl.CommandImpl;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link CodeLens}.
 */
@SuppressWarnings("all")
public class CodeLensBuilder implements IBuilder<CodeLens> {
  public CodeLensBuilder() {
    
  }
  
  public CodeLensBuilder(final Procedure1<CodeLensBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The range in which this code lens is valid. Should only span a single line.
   */
  private RangeImpl range;
  
  public CodeLensBuilder range(final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    this.range = (RangeImpl) range;
    return this;
  }
  
  public CodeLensBuilder range(final Procedure1<RangeBuilder> initializer) {
    RangeBuilder builder = new RangeBuilder();
    initializer.apply(builder);
    return range(builder.build());
  }
  
  public CodeLensBuilder range(final Position start, final Position end) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    Range instance = new RangeImpl((PositionImpl) start, (PositionImpl) end);
    return range(instance);
  }
  
  /**
   * The command this code lens represents.
   */
  private CommandImpl command;
  
  public CodeLensBuilder command(final Command command) {
    if (command != null && !(command instanceof CommandImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + command.getClass().getSimpleName());
    this.command = (CommandImpl) command;
    return this;
  }
  
  public CodeLensBuilder command(final Procedure1<CommandBuilder> initializer) {
    CommandBuilder builder = new CommandBuilder();
    initializer.apply(builder);
    return command(builder.build());
  }
  
  /**
   * An data entry field that is preserved on a code lens item between a code lens and a code lens resolve request.
   */
  private Object data;
  
  public CodeLensBuilder data(final Object data) {
    this.data = data;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public CodeLens build() {
    CodeLensImpl result = new CodeLensImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final CodeLensImpl result) {
    result.setRange(this.range);
    result.setCommand(this.command);
    result.setData(this.data);
  }
}
