/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.CompletionItem;
import io.typefox.lsapi.CompletionItemKind;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.TextEdit;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.TextEditBuilder;
import io.typefox.lsapi.impl.CompletionItemImpl;
import io.typefox.lsapi.impl.RangeImpl;
import io.typefox.lsapi.impl.TextEditImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link CompletionItem}.
 */
@SuppressWarnings("all")
public class CompletionItemBuilder implements IBuilder<CompletionItem> {
  public CompletionItemBuilder() {
    
  }
  
  public CompletionItemBuilder(final Procedure1<CompletionItemBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The label of this completion item. By default also the text that is inserted when selecting this completion.
   */
  private String label;
  
  public CompletionItemBuilder label(final String label) {
    this.label = label;
    return this;
  }
  
  /**
   * The kind of this completion item. Based of the kind an icon is chosen by the editor.
   */
  private CompletionItemKind kind;
  
  public CompletionItemBuilder kind(final CompletionItemKind kind) {
    this.kind = kind;
    return this;
  }
  
  /**
   * A human-readable string with additional information about this item, like type or symbol information.
   */
  private String detail;
  
  public CompletionItemBuilder detail(final String detail) {
    this.detail = detail;
    return this;
  }
  
  /**
   * A human-readable string that represents a doc-comment.
   */
  private String documentation;
  
  public CompletionItemBuilder documentation(final String documentation) {
    this.documentation = documentation;
    return this;
  }
  
  /**
   * A string that shoud be used when comparing this item with other items. When `falsy` the label is used.
   */
  private String sortText;
  
  public CompletionItemBuilder sortText(final String sortText) {
    this.sortText = sortText;
    return this;
  }
  
  /**
   * A string that should be used when filtering a set of completion items. When `falsy` the label is used.
   */
  private String filterText;
  
  public CompletionItemBuilder filterText(final String filterText) {
    this.filterText = filterText;
    return this;
  }
  
  /**
   * A string that should be inserted a document when selecting this completion. When `falsy` the label is used.
   */
  private String insertText;
  
  public CompletionItemBuilder insertText(final String insertText) {
    this.insertText = insertText;
    return this;
  }
  
  /**
   * An edit which is applied to a document when selecting this completion. When an edit is provided the value of
   * insertText is ignored.
   */
  private TextEditImpl textEdit;
  
  public CompletionItemBuilder textEdit(final TextEdit textEdit) {
    if (textEdit != null && !(textEdit instanceof TextEditImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textEdit.getClass().getSimpleName());
    this.textEdit = (TextEditImpl) textEdit;
    return this;
  }
  
  public CompletionItemBuilder textEdit(final Procedure1<TextEditBuilder> initializer) {
    TextEditBuilder builder = new TextEditBuilder();
    initializer.apply(builder);
    return textEdit(builder.build());
  }
  
  public CompletionItemBuilder textEdit(final Range range, final String newText) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    TextEdit instance = new TextEditImpl((RangeImpl) range, newText);
    return textEdit(instance);
  }
  
  /**
   * An data entry field that is preserved on a completion item between a completion and a completion resolve request.
   */
  private Object data;
  
  public CompletionItemBuilder data(final Object data) {
    this.data = data;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public CompletionItem build() {
    CompletionItemImpl result = new CompletionItemImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final CompletionItemImpl result) {
    result.setLabel(this.label);
    result.setKind(this.kind);
    result.setDetail(this.detail);
    result.setDocumentation(this.documentation);
    result.setSortText(this.sortText);
    result.setFilterText(this.filterText);
    result.setInsertText(this.insertText);
    result.setTextEdit(this.textEdit);
    result.setData(this.data);
  }
}
