/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Diagnostic;
import io.typefox.lsapi.DiagnosticSeverity;
import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.RangeBuilder;
import io.typefox.lsapi.impl.DiagnosticImpl;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link Diagnostic}.
 */
@SuppressWarnings("all")
public class DiagnosticBuilder implements IBuilder<Diagnostic> {
  public DiagnosticBuilder() {
    
  }
  
  public DiagnosticBuilder(final Procedure1<DiagnosticBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The range at which the message applies
   */
  private RangeImpl range;
  
  public DiagnosticBuilder range(final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    this.range = (RangeImpl) range;
    return this;
  }
  
  public DiagnosticBuilder range(final Procedure1<RangeBuilder> initializer) {
    RangeBuilder builder = new RangeBuilder();
    initializer.apply(builder);
    return range(builder.build());
  }
  
  public DiagnosticBuilder range(final Position start, final Position end) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    Range instance = new RangeImpl((PositionImpl) start, (PositionImpl) end);
    return range(instance);
  }
  
  /**
   * The diagnostic's severity. Can be omitted. If omitted it is up to the client to interpret diagnostics as error,
   * warning, info or hint.
   */
  private DiagnosticSeverity severity;
  
  public DiagnosticBuilder severity(final DiagnosticSeverity severity) {
    this.severity = severity;
    return this;
  }
  
  /**
   * The diagnostic's code. Can be omitted.
   */
  private String code;
  
  public DiagnosticBuilder code(final String code) {
    this.code = code;
    return this;
  }
  
  /**
   * A human-readable string describing the source of this diagnostic, e.g. 'typescript' or 'super lint'.
   */
  private String source;
  
  public DiagnosticBuilder source(final String source) {
    this.source = source;
    return this;
  }
  
  /**
   * The diagnostic's message.
   */
  private String message;
  
  public DiagnosticBuilder message(final String message) {
    this.message = message;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public Diagnostic build() {
    DiagnosticImpl result = new DiagnosticImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DiagnosticImpl result) {
    result.setRange(this.range);
    result.setSeverity(this.severity);
    result.setCode(this.code);
    result.setSource(this.source);
    result.setMessage(this.message);
  }
}
