/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.DidChangeTextDocumentParams;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.TextDocumentContentChangeEvent;
import io.typefox.lsapi.VersionedTextDocumentIdentifier;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.TextDocumentContentChangeEventBuilder;
import io.typefox.lsapi.builders.VersionedTextDocumentIdentifierBuilder;
import io.typefox.lsapi.impl.DidChangeTextDocumentParamsImpl;
import io.typefox.lsapi.impl.RangeImpl;
import io.typefox.lsapi.impl.TextDocumentContentChangeEventImpl;
import io.typefox.lsapi.impl.VersionedTextDocumentIdentifierImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link DidChangeTextDocumentParams}.
 */
@SuppressWarnings("all")
public class DidChangeTextDocumentParamsBuilder implements IBuilder<DidChangeTextDocumentParams> {
  public DidChangeTextDocumentParamsBuilder() {
    
  }
  
  public DidChangeTextDocumentParamsBuilder(final Procedure1<DidChangeTextDocumentParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The document that did change. The version number points to the version after all provided content changes have
   * been applied.
   */
  private VersionedTextDocumentIdentifierImpl textDocument;
  
  public DidChangeTextDocumentParamsBuilder textDocument(final VersionedTextDocumentIdentifier textDocument) {
    if (textDocument != null && !(textDocument instanceof VersionedTextDocumentIdentifierImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (VersionedTextDocumentIdentifierImpl) textDocument;
    return this;
  }
  
  public DidChangeTextDocumentParamsBuilder textDocument(final Procedure1<VersionedTextDocumentIdentifierBuilder> initializer) {
    VersionedTextDocumentIdentifierBuilder builder = new VersionedTextDocumentIdentifierBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  /**
   * Legacy property to support protocol version 1.0 requests.
   */
  private String uri;
  
  public DidChangeTextDocumentParamsBuilder uri(final String uri) {
    this.uri = uri;
    return this;
  }
  
  /**
   * The actual content changes.
   */
  private List<TextDocumentContentChangeEventImpl> contentChanges = Lists.newArrayList();
  
  public DidChangeTextDocumentParamsBuilder contentChange(final TextDocumentContentChangeEvent contentChange) {
    if (contentChange != null && !(contentChange instanceof TextDocumentContentChangeEventImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + contentChange.getClass().getSimpleName());
    this.contentChanges.add((TextDocumentContentChangeEventImpl) contentChange);
    return this;
  }
  
  public DidChangeTextDocumentParamsBuilder contentChange(final Procedure1<TextDocumentContentChangeEventBuilder> initializer) {
    TextDocumentContentChangeEventBuilder builder = new TextDocumentContentChangeEventBuilder();
    initializer.apply(builder);
    return contentChange(builder.build());
  }
  
  public DidChangeTextDocumentParamsBuilder contentChange(final Range range, final Integer rangeLength, final String text) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    TextDocumentContentChangeEvent instance = new TextDocumentContentChangeEventImpl((RangeImpl) range, rangeLength, text);
    return contentChange(instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public DidChangeTextDocumentParams build() {
    DidChangeTextDocumentParamsImpl result = new DidChangeTextDocumentParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DidChangeTextDocumentParamsImpl result) {
    result.setTextDocument(this.textDocument);
    result.setUri(this.uri);
    result.setContentChanges(this.contentChanges);
  }
}
