/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.DocumentOnTypeFormattingParams;
import io.typefox.lsapi.FormattingOptions;
import io.typefox.lsapi.Position;
import io.typefox.lsapi.TextDocumentIdentifier;
import io.typefox.lsapi.builders.FormattingOptionsBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.PositionBuilder;
import io.typefox.lsapi.builders.TextDocumentIdentifierBuilder;
import io.typefox.lsapi.impl.DocumentOnTypeFormattingParamsImpl;
import io.typefox.lsapi.impl.FormattingOptionsImpl;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link DocumentOnTypeFormattingParams}.
 */
@SuppressWarnings("all")
public class DocumentOnTypeFormattingParamsBuilder implements IBuilder<DocumentOnTypeFormattingParams> {
  public DocumentOnTypeFormattingParamsBuilder() {
    
  }
  
  public DocumentOnTypeFormattingParamsBuilder(final Procedure1<DocumentOnTypeFormattingParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The position at which this request was send.
   */
  private PositionImpl position;
  
  public DocumentOnTypeFormattingParamsBuilder position(final Position position) {
    if (position != null && !(position instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + position.getClass().getSimpleName());
    this.position = (PositionImpl) position;
    return this;
  }
  
  public DocumentOnTypeFormattingParamsBuilder position(final Procedure1<PositionBuilder> initializer) {
    PositionBuilder builder = new PositionBuilder();
    initializer.apply(builder);
    return position(builder.build());
  }
  
  public DocumentOnTypeFormattingParamsBuilder position(final int line, final int character) {
    Position instance = new PositionImpl(line, character);
    return position(instance);
  }
  
  /**
   * The character that has been typed.
   */
  private String ch;
  
  public DocumentOnTypeFormattingParamsBuilder ch(final String ch) {
    this.ch = ch;
    return this;
  }
  
  /**
   * The document to format.
   */
  private TextDocumentIdentifierImpl textDocument;
  
  public DocumentOnTypeFormattingParamsBuilder textDocument(final TextDocumentIdentifier textDocument) {
    if (textDocument != null && !(textDocument instanceof TextDocumentIdentifierImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (TextDocumentIdentifierImpl) textDocument;
    return this;
  }
  
  public DocumentOnTypeFormattingParamsBuilder textDocument(final Procedure1<TextDocumentIdentifierBuilder> initializer) {
    TextDocumentIdentifierBuilder builder = new TextDocumentIdentifierBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  public DocumentOnTypeFormattingParamsBuilder textDocument(final String uri) {
    TextDocumentIdentifier instance = new TextDocumentIdentifierImpl(uri);
    return textDocument(instance);
  }
  
  /**
   * The format options
   */
  private FormattingOptionsImpl options;
  
  public DocumentOnTypeFormattingParamsBuilder options(final FormattingOptions options) {
    if (options != null && !(options instanceof FormattingOptionsImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + options.getClass().getSimpleName());
    this.options = (FormattingOptionsImpl) options;
    return this;
  }
  
  public DocumentOnTypeFormattingParamsBuilder options(final Procedure1<FormattingOptionsBuilder> initializer) {
    FormattingOptionsBuilder builder = new FormattingOptionsBuilder();
    initializer.apply(builder);
    return options(builder.build());
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public DocumentOnTypeFormattingParams build() {
    DocumentOnTypeFormattingParamsImpl result = new DocumentOnTypeFormattingParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DocumentOnTypeFormattingParamsImpl result) {
    result.setPosition(this.position);
    result.setCh(this.ch);
    result.setTextDocument(this.textDocument);
    result.setOptions(this.options);
  }
}
