/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.DocumentRangeFormattingParams;
import io.typefox.lsapi.FormattingOptions;
import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.TextDocumentIdentifier;
import io.typefox.lsapi.builders.FormattingOptionsBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.RangeBuilder;
import io.typefox.lsapi.builders.TextDocumentIdentifierBuilder;
import io.typefox.lsapi.impl.DocumentRangeFormattingParamsImpl;
import io.typefox.lsapi.impl.FormattingOptionsImpl;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link DocumentRangeFormattingParams}.
 */
@SuppressWarnings("all")
public class DocumentRangeFormattingParamsBuilder implements IBuilder<DocumentRangeFormattingParams> {
  public DocumentRangeFormattingParamsBuilder() {
    
  }
  
  public DocumentRangeFormattingParamsBuilder(final Procedure1<DocumentRangeFormattingParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The range to format
   */
  private RangeImpl range;
  
  public DocumentRangeFormattingParamsBuilder range(final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    this.range = (RangeImpl) range;
    return this;
  }
  
  public DocumentRangeFormattingParamsBuilder range(final Procedure1<RangeBuilder> initializer) {
    RangeBuilder builder = new RangeBuilder();
    initializer.apply(builder);
    return range(builder.build());
  }
  
  public DocumentRangeFormattingParamsBuilder range(final Position start, final Position end) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    Range instance = new RangeImpl((PositionImpl) start, (PositionImpl) end);
    return range(instance);
  }
  
  /**
   * The document to format.
   */
  private TextDocumentIdentifierImpl textDocument;
  
  public DocumentRangeFormattingParamsBuilder textDocument(final TextDocumentIdentifier textDocument) {
    if (textDocument != null && !(textDocument instanceof TextDocumentIdentifierImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (TextDocumentIdentifierImpl) textDocument;
    return this;
  }
  
  public DocumentRangeFormattingParamsBuilder textDocument(final Procedure1<TextDocumentIdentifierBuilder> initializer) {
    TextDocumentIdentifierBuilder builder = new TextDocumentIdentifierBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  public DocumentRangeFormattingParamsBuilder textDocument(final String uri) {
    TextDocumentIdentifier instance = new TextDocumentIdentifierImpl(uri);
    return textDocument(instance);
  }
  
  /**
   * The format options
   */
  private FormattingOptionsImpl options;
  
  public DocumentRangeFormattingParamsBuilder options(final FormattingOptions options) {
    if (options != null && !(options instanceof FormattingOptionsImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + options.getClass().getSimpleName());
    this.options = (FormattingOptionsImpl) options;
    return this;
  }
  
  public DocumentRangeFormattingParamsBuilder options(final Procedure1<FormattingOptionsBuilder> initializer) {
    FormattingOptionsBuilder builder = new FormattingOptionsBuilder();
    initializer.apply(builder);
    return options(builder.build());
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public DocumentRangeFormattingParams build() {
    DocumentRangeFormattingParamsImpl result = new DocumentRangeFormattingParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DocumentRangeFormattingParamsImpl result) {
    result.setRange(this.range);
    result.setTextDocument(this.textDocument);
    result.setOptions(this.options);
  }
}
