/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.Hover;
import io.typefox.lsapi.MarkedString;
import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.MarkedStringBuilder;
import io.typefox.lsapi.builders.RangeBuilder;
import io.typefox.lsapi.impl.HoverImpl;
import io.typefox.lsapi.impl.MarkedStringImpl;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link Hover}.
 */
@SuppressWarnings("all")
public class HoverBuilder implements IBuilder<Hover> {
  public HoverBuilder() {
    
  }
  
  public HoverBuilder(final Procedure1<HoverBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The hover's content
   */
  private List<MarkedStringImpl> contents = Lists.newArrayList();
  
  public HoverBuilder content(final MarkedString content) {
    if (content != null && !(content instanceof MarkedStringImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + content.getClass().getSimpleName());
    this.contents.add((MarkedStringImpl) content);
    return this;
  }
  
  public HoverBuilder content(final Procedure1<MarkedStringBuilder> initializer) {
    MarkedStringBuilder builder = new MarkedStringBuilder();
    initializer.apply(builder);
    return content(builder.build());
  }
  
  public HoverBuilder content(final String language, final String value) {
    MarkedString instance = new MarkedStringImpl(language, value);
    return content(instance);
  }
  
  /**
   * An optional range
   */
  private RangeImpl range;
  
  public HoverBuilder range(final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    this.range = (RangeImpl) range;
    return this;
  }
  
  public HoverBuilder range(final Procedure1<RangeBuilder> initializer) {
    RangeBuilder builder = new RangeBuilder();
    initializer.apply(builder);
    return range(builder.build());
  }
  
  public HoverBuilder range(final Position start, final Position end) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    Range instance = new RangeImpl((PositionImpl) start, (PositionImpl) end);
    return range(instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public Hover build() {
    HoverImpl result = new HoverImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final HoverImpl result) {
    result.setContents(this.contents);
    result.setRange(this.range);
  }
}
