/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Position;
import io.typefox.lsapi.ReferenceContext;
import io.typefox.lsapi.ReferenceParams;
import io.typefox.lsapi.TextDocumentIdentifier;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.PositionBuilder;
import io.typefox.lsapi.builders.ReferenceContextBuilder;
import io.typefox.lsapi.builders.TextDocumentIdentifierBuilder;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.ReferenceContextImpl;
import io.typefox.lsapi.impl.ReferenceParamsImpl;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link ReferenceParams}.
 */
@SuppressWarnings("all")
public class ReferenceParamsBuilder implements IBuilder<ReferenceParams> {
  public ReferenceParamsBuilder() {
    
  }
  
  public ReferenceParamsBuilder(final Procedure1<ReferenceParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  private ReferenceContextImpl context;
  
  public ReferenceParamsBuilder context(final ReferenceContext context) {
    if (context != null && !(context instanceof ReferenceContextImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + context.getClass().getSimpleName());
    this.context = (ReferenceContextImpl) context;
    return this;
  }
  
  public ReferenceParamsBuilder context(final Procedure1<ReferenceContextBuilder> initializer) {
    ReferenceContextBuilder builder = new ReferenceContextBuilder();
    initializer.apply(builder);
    return context(builder.build());
  }
  
  public ReferenceParamsBuilder context(final boolean includeDeclaration) {
    ReferenceContext instance = new ReferenceContextImpl(includeDeclaration);
    return context(instance);
  }
  
  /**
   * The text document.
   */
  private TextDocumentIdentifierImpl textDocument;
  
  public ReferenceParamsBuilder textDocument(final TextDocumentIdentifier textDocument) {
    if (textDocument != null && !(textDocument instanceof TextDocumentIdentifierImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (TextDocumentIdentifierImpl) textDocument;
    return this;
  }
  
  public ReferenceParamsBuilder textDocument(final Procedure1<TextDocumentIdentifierBuilder> initializer) {
    TextDocumentIdentifierBuilder builder = new TextDocumentIdentifierBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  public ReferenceParamsBuilder textDocument(final String uri) {
    TextDocumentIdentifier instance = new TextDocumentIdentifierImpl(uri);
    return textDocument(instance);
  }
  
  /**
   * Legacy property to support protocol version 1.0 requests.
   */
  private String uri;
  
  public ReferenceParamsBuilder uri(final String uri) {
    this.uri = uri;
    return this;
  }
  
  /**
   * The position inside the text document.
   */
  private PositionImpl position;
  
  public ReferenceParamsBuilder position(final Position position) {
    if (position != null && !(position instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + position.getClass().getSimpleName());
    this.position = (PositionImpl) position;
    return this;
  }
  
  public ReferenceParamsBuilder position(final Procedure1<PositionBuilder> initializer) {
    PositionBuilder builder = new PositionBuilder();
    initializer.apply(builder);
    return position(builder.build());
  }
  
  public ReferenceParamsBuilder position(final int line, final int character) {
    Position instance = new PositionImpl(line, character);
    return position(instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public ReferenceParams build() {
    ReferenceParamsImpl result = new ReferenceParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final ReferenceParamsImpl result) {
    result.setContext(this.context);
    result.setTextDocument(this.textDocument);
    result.setUri(this.uri);
    result.setPosition(this.position);
  }
}
