/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Position;
import io.typefox.lsapi.RenameParams;
import io.typefox.lsapi.TextDocumentIdentifier;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.PositionBuilder;
import io.typefox.lsapi.builders.TextDocumentIdentifierBuilder;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RenameParamsImpl;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link RenameParams}.
 */
@SuppressWarnings("all")
public class RenameParamsBuilder implements IBuilder<RenameParams> {
  public RenameParamsBuilder() {
    
  }
  
  public RenameParamsBuilder(final Procedure1<RenameParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The document in which to find the symbol.
   */
  private TextDocumentIdentifierImpl textDocument;
  
  public RenameParamsBuilder textDocument(final TextDocumentIdentifier textDocument) {
    if (textDocument != null && !(textDocument instanceof TextDocumentIdentifierImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (TextDocumentIdentifierImpl) textDocument;
    return this;
  }
  
  public RenameParamsBuilder textDocument(final Procedure1<TextDocumentIdentifierBuilder> initializer) {
    TextDocumentIdentifierBuilder builder = new TextDocumentIdentifierBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  public RenameParamsBuilder textDocument(final String uri) {
    TextDocumentIdentifier instance = new TextDocumentIdentifierImpl(uri);
    return textDocument(instance);
  }
  
  /**
   * The position at which this request was send.
   */
  private PositionImpl position;
  
  public RenameParamsBuilder position(final Position position) {
    if (position != null && !(position instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + position.getClass().getSimpleName());
    this.position = (PositionImpl) position;
    return this;
  }
  
  public RenameParamsBuilder position(final Procedure1<PositionBuilder> initializer) {
    PositionBuilder builder = new PositionBuilder();
    initializer.apply(builder);
    return position(builder.build());
  }
  
  public RenameParamsBuilder position(final int line, final int character) {
    Position instance = new PositionImpl(line, character);
    return position(instance);
  }
  
  /**
   * The new name of the symbol. If the given name is not valid the request must return a
   * ResponseError with an appropriate message set.
   */
  private String newName;
  
  public RenameParamsBuilder newName(final String newName) {
    this.newName = newName;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public RenameParams build() {
    RenameParamsImpl result = new RenameParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final RenameParamsImpl result) {
    result.setTextDocument(this.textDocument);
    result.setPosition(this.position);
    result.setNewName(this.newName);
  }
}
