/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.CodeLensOptions;
import io.typefox.lsapi.CompletionOptions;
import io.typefox.lsapi.DocumentOnTypeFormattingOptions;
import io.typefox.lsapi.ServerCapabilities;
import io.typefox.lsapi.SignatureHelpOptions;
import io.typefox.lsapi.TextDocumentSyncKind;
import io.typefox.lsapi.builders.CodeLensOptionsBuilder;
import io.typefox.lsapi.builders.CompletionOptionsBuilder;
import io.typefox.lsapi.builders.DocumentOnTypeFormattingOptionsBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.SignatureHelpOptionsBuilder;
import io.typefox.lsapi.impl.CodeLensOptionsImpl;
import io.typefox.lsapi.impl.CompletionOptionsImpl;
import io.typefox.lsapi.impl.DocumentOnTypeFormattingOptionsImpl;
import io.typefox.lsapi.impl.ServerCapabilitiesImpl;
import io.typefox.lsapi.impl.SignatureHelpOptionsImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link ServerCapabilities}.
 */
@SuppressWarnings("all")
public class ServerCapabilitiesBuilder implements IBuilder<ServerCapabilities> {
  public ServerCapabilitiesBuilder() {
    
  }
  
  public ServerCapabilitiesBuilder(final Procedure1<ServerCapabilitiesBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * Defines how text documents are synced.
   */
  private TextDocumentSyncKind textDocumentSync;
  
  public ServerCapabilitiesBuilder textDocumentSync(final TextDocumentSyncKind textDocumentSync) {
    this.textDocumentSync = textDocumentSync;
    return this;
  }
  
  /**
   * The server provides hover support.
   */
  private Boolean hoverProvider;
  
  public ServerCapabilitiesBuilder hoverProvider(final Boolean hoverProvider) {
    this.hoverProvider = hoverProvider;
    return this;
  }
  
  /**
   * The server provides completion support.
   */
  private CompletionOptionsImpl completionProvider;
  
  public ServerCapabilitiesBuilder completionProvider(final CompletionOptions completionProvider) {
    if (completionProvider != null && !(completionProvider instanceof CompletionOptionsImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + completionProvider.getClass().getSimpleName());
    this.completionProvider = (CompletionOptionsImpl) completionProvider;
    return this;
  }
  
  public ServerCapabilitiesBuilder completionProvider(final Procedure1<CompletionOptionsBuilder> initializer) {
    CompletionOptionsBuilder builder = new CompletionOptionsBuilder();
    initializer.apply(builder);
    return completionProvider(builder.build());
  }
  
  /**
   * The server provides signature help support.
   */
  private SignatureHelpOptionsImpl signatureHelpProvider;
  
  public ServerCapabilitiesBuilder signatureHelpProvider(final SignatureHelpOptions signatureHelpProvider) {
    if (signatureHelpProvider != null && !(signatureHelpProvider instanceof SignatureHelpOptionsImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + signatureHelpProvider.getClass().getSimpleName());
    this.signatureHelpProvider = (SignatureHelpOptionsImpl) signatureHelpProvider;
    return this;
  }
  
  public ServerCapabilitiesBuilder signatureHelpProvider(final Procedure1<SignatureHelpOptionsBuilder> initializer) {
    SignatureHelpOptionsBuilder builder = new SignatureHelpOptionsBuilder();
    initializer.apply(builder);
    return signatureHelpProvider(builder.build());
  }
  
  /**
   * The server provides goto definition support.
   */
  private Boolean definitionProvider;
  
  public ServerCapabilitiesBuilder definitionProvider(final Boolean definitionProvider) {
    this.definitionProvider = definitionProvider;
    return this;
  }
  
  /**
   * The server provides find references support.
   */
  private Boolean referencesProvider;
  
  public ServerCapabilitiesBuilder referencesProvider(final Boolean referencesProvider) {
    this.referencesProvider = referencesProvider;
    return this;
  }
  
  /**
   * The server provides document highlight support.
   */
  private Boolean documentHighlightProvider;
  
  public ServerCapabilitiesBuilder documentHighlightProvider(final Boolean documentHighlightProvider) {
    this.documentHighlightProvider = documentHighlightProvider;
    return this;
  }
  
  /**
   * The server provides document symbol support.
   */
  private Boolean documentSymbolProvider;
  
  public ServerCapabilitiesBuilder documentSymbolProvider(final Boolean documentSymbolProvider) {
    this.documentSymbolProvider = documentSymbolProvider;
    return this;
  }
  
  /**
   * The server provides workspace symbol support.
   */
  private Boolean workspaceSymbolProvider;
  
  public ServerCapabilitiesBuilder workspaceSymbolProvider(final Boolean workspaceSymbolProvider) {
    this.workspaceSymbolProvider = workspaceSymbolProvider;
    return this;
  }
  
  /**
   * The server provides code actions.
   */
  private Boolean codeActionProvider;
  
  public ServerCapabilitiesBuilder codeActionProvider(final Boolean codeActionProvider) {
    this.codeActionProvider = codeActionProvider;
    return this;
  }
  
  /**
   * The server provides code lens.
   */
  private CodeLensOptionsImpl codeLensProvider;
  
  public ServerCapabilitiesBuilder codeLensProvider(final CodeLensOptions codeLensProvider) {
    if (codeLensProvider != null && !(codeLensProvider instanceof CodeLensOptionsImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + codeLensProvider.getClass().getSimpleName());
    this.codeLensProvider = (CodeLensOptionsImpl) codeLensProvider;
    return this;
  }
  
  public ServerCapabilitiesBuilder codeLensProvider(final Procedure1<CodeLensOptionsBuilder> initializer) {
    CodeLensOptionsBuilder builder = new CodeLensOptionsBuilder();
    initializer.apply(builder);
    return codeLensProvider(builder.build());
  }
  
  public ServerCapabilitiesBuilder codeLensProvider(final boolean resolveProvider) {
    CodeLensOptions instance = new CodeLensOptionsImpl(resolveProvider);
    return codeLensProvider(instance);
  }
  
  /**
   * The server provides document formatting.
   */
  private Boolean documentFormattingProvider;
  
  public ServerCapabilitiesBuilder documentFormattingProvider(final Boolean documentFormattingProvider) {
    this.documentFormattingProvider = documentFormattingProvider;
    return this;
  }
  
  /**
   * The server provides document range formatting.
   */
  private Boolean documentRangeFormattingProvider;
  
  public ServerCapabilitiesBuilder documentRangeFormattingProvider(final Boolean documentRangeFormattingProvider) {
    this.documentRangeFormattingProvider = documentRangeFormattingProvider;
    return this;
  }
  
  /**
   * The server provides document formatting on typing.
   */
  private DocumentOnTypeFormattingOptionsImpl documentOnTypeFormattingProvider;
  
  public ServerCapabilitiesBuilder documentOnTypeFormattingProvider(final DocumentOnTypeFormattingOptions documentOnTypeFormattingProvider) {
    if (documentOnTypeFormattingProvider != null && !(documentOnTypeFormattingProvider instanceof DocumentOnTypeFormattingOptionsImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + documentOnTypeFormattingProvider.getClass().getSimpleName());
    this.documentOnTypeFormattingProvider = (DocumentOnTypeFormattingOptionsImpl) documentOnTypeFormattingProvider;
    return this;
  }
  
  public ServerCapabilitiesBuilder documentOnTypeFormattingProvider(final Procedure1<DocumentOnTypeFormattingOptionsBuilder> initializer) {
    DocumentOnTypeFormattingOptionsBuilder builder = new DocumentOnTypeFormattingOptionsBuilder();
    initializer.apply(builder);
    return documentOnTypeFormattingProvider(builder.build());
  }
  
  /**
   * The server provides rename support.
   */
  private Boolean renameProvider;
  
  public ServerCapabilitiesBuilder renameProvider(final Boolean renameProvider) {
    this.renameProvider = renameProvider;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public ServerCapabilities build() {
    ServerCapabilitiesImpl result = new ServerCapabilitiesImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final ServerCapabilitiesImpl result) {
    result.setTextDocumentSync(this.textDocumentSync);
    result.setHoverProvider(this.hoverProvider);
    result.setCompletionProvider(this.completionProvider);
    result.setSignatureHelpProvider(this.signatureHelpProvider);
    result.setDefinitionProvider(this.definitionProvider);
    result.setReferencesProvider(this.referencesProvider);
    result.setDocumentHighlightProvider(this.documentHighlightProvider);
    result.setDocumentSymbolProvider(this.documentSymbolProvider);
    result.setWorkspaceSymbolProvider(this.workspaceSymbolProvider);
    result.setCodeActionProvider(this.codeActionProvider);
    result.setCodeLensProvider(this.codeLensProvider);
    result.setDocumentFormattingProvider(this.documentFormattingProvider);
    result.setDocumentRangeFormattingProvider(this.documentRangeFormattingProvider);
    result.setDocumentOnTypeFormattingProvider(this.documentOnTypeFormattingProvider);
    result.setRenameProvider(this.renameProvider);
  }
}
