/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.ParameterInformation;
import io.typefox.lsapi.SignatureInformation;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.ParameterInformationBuilder;
import io.typefox.lsapi.impl.ParameterInformationImpl;
import io.typefox.lsapi.impl.SignatureInformationImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link SignatureInformation}.
 */
@SuppressWarnings("all")
public class SignatureInformationBuilder implements IBuilder<SignatureInformation> {
  public SignatureInformationBuilder() {
    
  }
  
  public SignatureInformationBuilder(final Procedure1<SignatureInformationBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The label of this signature. Will be shown in the UI.
   */
  private String label;
  
  public SignatureInformationBuilder label(final String label) {
    this.label = label;
    return this;
  }
  
  /**
   * The human-readable doc-comment of this signature. Will be shown in the UI but can be omitted.
   */
  private String documentation;
  
  public SignatureInformationBuilder documentation(final String documentation) {
    this.documentation = documentation;
    return this;
  }
  
  /**
   * The parameters of this signature.
   */
  private List<ParameterInformationImpl> parameters = Lists.newArrayList();
  
  public SignatureInformationBuilder parameter(final ParameterInformation parameter) {
    if (parameter != null && !(parameter instanceof ParameterInformationImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + parameter.getClass().getSimpleName());
    this.parameters.add((ParameterInformationImpl) parameter);
    return this;
  }
  
  public SignatureInformationBuilder parameter(final Procedure1<ParameterInformationBuilder> initializer) {
    ParameterInformationBuilder builder = new ParameterInformationBuilder();
    initializer.apply(builder);
    return parameter(builder.build());
  }
  
  public SignatureInformationBuilder parameter(final String label, final String documentation) {
    ParameterInformation instance = new ParameterInformationImpl(label, documentation);
    return parameter(instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public SignatureInformation build() {
    SignatureInformationImpl result = new SignatureInformationImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final SignatureInformationImpl result) {
    result.setLabel(this.label);
    result.setDocumentation(this.documentation);
    result.setParameters(this.parameters);
  }
}
