/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Location;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.SymbolInformation;
import io.typefox.lsapi.SymbolKind;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.LocationBuilder;
import io.typefox.lsapi.impl.LocationImpl;
import io.typefox.lsapi.impl.RangeImpl;
import io.typefox.lsapi.impl.SymbolInformationImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link SymbolInformation}.
 */
@SuppressWarnings("all")
public class SymbolInformationBuilder implements IBuilder<SymbolInformation> {
  public SymbolInformationBuilder() {
    
  }
  
  public SymbolInformationBuilder(final Procedure1<SymbolInformationBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The name of this symbol.
   */
  private String name;
  
  public SymbolInformationBuilder name(final String name) {
    this.name = name;
    return this;
  }
  
  /**
   * The kind of this symbol.
   */
  private SymbolKind kind;
  
  public SymbolInformationBuilder kind(final SymbolKind kind) {
    this.kind = kind;
    return this;
  }
  
  /**
   * The location of this symbol.
   */
  private LocationImpl location;
  
  public SymbolInformationBuilder location(final Location location) {
    if (location != null && !(location instanceof LocationImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + location.getClass().getSimpleName());
    this.location = (LocationImpl) location;
    return this;
  }
  
  public SymbolInformationBuilder location(final Procedure1<LocationBuilder> initializer) {
    LocationBuilder builder = new LocationBuilder();
    initializer.apply(builder);
    return location(builder.build());
  }
  
  public SymbolInformationBuilder location(final String uri, final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    Location instance = new LocationImpl(uri, (RangeImpl) range);
    return location(instance);
  }
  
  /**
   * The name of the symbol containing this symbol.
   */
  private String containerName;
  
  public SymbolInformationBuilder containerName(final String containerName) {
    this.containerName = containerName;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public SymbolInformation build() {
    SymbolInformationImpl result = new SymbolInformationImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final SymbolInformationImpl result) {
    result.setName(this.name);
    result.setKind(this.kind);
    result.setLocation(this.location);
    result.setContainerName(this.containerName);
  }
}
