/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Position;
import io.typefox.lsapi.TextDocumentIdentifier;
import io.typefox.lsapi.TextDocumentPositionParams;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.PositionBuilder;
import io.typefox.lsapi.builders.TextDocumentIdentifierBuilder;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import io.typefox.lsapi.impl.TextDocumentPositionParamsImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link TextDocumentPositionParams}.
 */
@SuppressWarnings("all")
public class TextDocumentPositionParamsBuilder implements IBuilder<TextDocumentPositionParams> {
  public TextDocumentPositionParamsBuilder() {
    
  }
  
  public TextDocumentPositionParamsBuilder(final Procedure1<TextDocumentPositionParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The text document.
   */
  private TextDocumentIdentifierImpl textDocument;
  
  public TextDocumentPositionParamsBuilder textDocument(final TextDocumentIdentifier textDocument) {
    if (textDocument != null && !(textDocument instanceof TextDocumentIdentifierImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (TextDocumentIdentifierImpl) textDocument;
    return this;
  }
  
  public TextDocumentPositionParamsBuilder textDocument(final Procedure1<TextDocumentIdentifierBuilder> initializer) {
    TextDocumentIdentifierBuilder builder = new TextDocumentIdentifierBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  public TextDocumentPositionParamsBuilder textDocument(final String uri) {
    TextDocumentIdentifier instance = new TextDocumentIdentifierImpl(uri);
    return textDocument(instance);
  }
  
  /**
   * Legacy property to support protocol version 1.0 requests.
   */
  private String uri;
  
  public TextDocumentPositionParamsBuilder uri(final String uri) {
    this.uri = uri;
    return this;
  }
  
  /**
   * The position inside the text document.
   */
  private PositionImpl position;
  
  public TextDocumentPositionParamsBuilder position(final Position position) {
    if (position != null && !(position instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + position.getClass().getSimpleName());
    this.position = (PositionImpl) position;
    return this;
  }
  
  public TextDocumentPositionParamsBuilder position(final Procedure1<PositionBuilder> initializer) {
    PositionBuilder builder = new PositionBuilder();
    initializer.apply(builder);
    return position(builder.build());
  }
  
  public TextDocumentPositionParamsBuilder position(final int line, final int character) {
    Position instance = new PositionImpl(line, character);
    return position(instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public TextDocumentPositionParams build() {
    TextDocumentPositionParamsImpl result = new TextDocumentPositionParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final TextDocumentPositionParamsImpl result) {
    result.setTextDocument(this.textDocument);
    result.setUri(this.uri);
    result.setPosition(this.position);
  }
}
