/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.impl;

import io.typefox.lsapi.Command;
import java.util.List;
import javax.annotation.Nullable;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

/**
 * Represents a reference to a command. Provides a title which will be used to represent a command in the UI and,
 * optionally, an array of arguments which will be passed to the command handler function when invoked.
 */
@SuppressWarnings("all")
public class CommandImpl implements Command {
  /**
   * Title of the command, like `save`.
   */
  private String title;
  
  @Pure
  @Override
  public String getTitle() {
    return this.title;
  }
  
  public void setTitle(final String title) {
    this.title = title;
  }
  
  /**
   * The identifier of the actual command handler.
   */
  private String command;
  
  @Pure
  @Override
  public String getCommand() {
    return this.command;
  }
  
  public void setCommand(final String command) {
    this.command = command;
  }
  
  /**
   * Arguments that the command handler should be invoked with.
   */
  @Nullable
  private List<Object> arguments;
  
  @Pure
  @Override
  @Nullable
  public List<Object> getArguments() {
    return this.arguments;
  }
  
  public void setArguments(@Nullable final List<Object> arguments) {
    this.arguments = arguments;
  }
  
  public CommandImpl() {
    
  }
  
  public CommandImpl(final CommandImpl other) {
    this.title = other.title;
    this.command = other.command;
    this.arguments = other.arguments;
  }
  
  public CommandImpl(final String title, final String command, final List<Object> arguments) {
    this.title = title;
    this.command = command;
    this.arguments = arguments;
  }
  
  @Override
  @Pure
  public String toString() {
    ToStringBuilder b = new ToStringBuilder(this);
    b.add("title", this.title);
    b.add("command", this.command);
    b.add("arguments", this.arguments);
    return b.toString();
  }
  
  @Override
  @Pure
  public boolean equals(final Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    CommandImpl other = (CommandImpl) obj;
    if (this.title == null) {
      if (other.title != null)
        return false;
    } else if (!this.title.equals(other.title))
      return false;
    if (this.command == null) {
      if (other.command != null)
        return false;
    } else if (!this.command.equals(other.command))
      return false;
    if (this.arguments == null) {
      if (other.arguments != null)
        return false;
    } else if (!this.arguments.equals(other.arguments))
      return false;
    return true;
  }
  
  @Override
  @Pure
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((this.title== null) ? 0 : this.title.hashCode());
    result = prime * result + ((this.command== null) ? 0 : this.command.hashCode());
    result = prime * result + ((this.arguments== null) ? 0 : this.arguments.hashCode());
    return result;
  }
}
